@@ -10,7 +10,7 @@ void main()
 
   for(i = 1; i <= 10000; i++)
     {
-       printf("grp%d:*:%d:",i,i);
+       printf("grp%d:*:%d:",i,6000+i);
        for(j = 1; j <= 20; j++) printf("user%d,",j);
        printf("user4000\n");
     }
