@@ -1,7 +1,7 @@
 /*
 ** server.c - YP server routines.
 **
-** Copyright (c) 1996, 1997 Thorsten Kukuk
+** Copyright (c) 1996, 1997, 1998 Thorsten Kukuk
 ** Copyright (c) 1993 Signum Support AB, Sweden
 **
 ** This file is part of the NYS YP Server.
@@ -26,13 +26,15 @@
 */
 
 #ifndef LINT
-static const char rcsid[] = "$Id: server.c,v 1.14 1997/04/12 16:16:42 kukuk Exp $";
+static const char rcsid[] = "$Id: server.c,v 1.24 1998/10/10 08:16:53 kukuk Exp $";
 #endif
 
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
+#define _GNU_SOURCE
+
 #include "system.h"
 
 #include <stdio.h>
@@ -63,6 +65,10 @@ int xfr_check_port = 1;
 #include <compat/strdup.c>
 #endif
 
+#if !defined(HAVE_STRNDUP)
+#include <compat/strndup.c>
+#endif
+
 #if !defined(HAVE_GETHOSTNAME)
 #include <sys/utsname.h>
 
@@ -87,6 +93,60 @@ gethostname (char *buf, int buflen)
 
 #endif
 
+/* repleace "field" with x, if field 3 != user id and user id != 0 */
+static int
+mangle_field (datum * val, int field)
+{
+  int i, j, k, anz;
+  char *p;
+
+  /* allocate a little bit more memory, it's safer, because the
+     field could be empty */
+  if ((p = malloc (val->dsize + 3)) == NULL)
+    {
+      yp_msg ("ERROR: could not allocate enough memory! [%s|%d]\n",
+              __FILE__, __LINE__);
+      return -1;
+    }
+
+  anz = 0;
+  for (i = 0; i < val->dsize; i++)
+    {
+      if (val->dptr[i] == ':')
+        anz++;
+      if (anz + 1 == field)
+        {
+          anz = i;
+          strncpy (p, val->dptr, anz);
+          p[anz] = 0;
+          /* if field == 1, we don't need a beginning ":" */
+          if (field == 1)
+            {
+              strcat (p, "x");
+              anz += 1;
+            }
+          else
+            {
+              strcat (p, ":x");
+              anz += 2;
+            }
+          for (j = anz; j < val->dsize && val->dptr[j] != ':'; j++);
+          for (k = j; k < val->dsize; k++)
+            {
+              p[anz] = val->dptr[k];
+              anz++;
+            }
+          free (val->dptr);
+          val->dptr = p;
+          val->dsize = anz;
+          return 0;
+        }
+    }
+  free (p);
+  return 0;
+}
+
+
 void *
 ypproc_null_2_svc (void *dummy, struct svc_req *rqstp)
 {
@@ -204,12 +264,12 @@ get_dtm (const char *domain, const char *map)
 ** YP function "MATCH" implementation
 */
 ypresp_val *
-ypproc_match_2_svc (ypreq_key * key, struct svc_req *rqstp)
+ypproc_match_2_svc (ypreq_key *key, struct svc_req *rqstp)
 {
   static ypresp_val result;
   struct sockaddr_in *rqhost;
   int valid;
-  int mangle_field = 0;
+  int mangle = 0;
 
   if (debug_flag)
     {
@@ -228,6 +288,13 @@ ypproc_match_2_svc (ypreq_key * key, struct svc_req *rqstp)
 	      key->key.keydat_val);
     }
 
+  result.val.valdat_len = 0;
+  if (result.val.valdat_val)
+    {
+      ypdb_free (result.val.valdat_val);
+      result.val.valdat_val = NULL;
+    }
+
   if ((valid = is_valid_host (rqstp, key->map, key->domain)) < 1)
     {
       if (valid == 0)
@@ -246,14 +313,7 @@ ypproc_match_2_svc (ypreq_key * key, struct svc_req *rqstp)
       return &result;
     }
   else if (valid > 200)
-    mangle_field = valid - 200;
-
-  result.val.valdat_len = 0;
-  if (result.val.valdat_val)
-    {
-      free (result.val.valdat_val);
-      result.val.valdat_val = NULL;
-    }
+    mangle = valid - 200;
 
   if (key->domain[0] == '\0' || key->map[0] == '\0' ||
       key->key.keydat_len == 0 || key->key.keydat_val[0] == '\0')
@@ -270,14 +330,19 @@ ypproc_match_2_svc (ypreq_key * key, struct svc_req *rqstp)
 	  qdat.dsize = key->key.keydat_len;
 	  qdat.dptr = key->key.keydat_val;
 
-	  result.stat = ypdb_read (dbp, &qdat, NULL, &rdat,
-				 F_ALL, mangle_field);
+	  rdat = ypdb_fetch (dbp, qdat);
 
-	  if (result.stat == YP_TRUE)
-	    {
-	      result.val.valdat_len = rdat.dsize;
-	      result.val.valdat_val = rdat.dptr;
-	    }
+          if (rdat.dptr != NULL)
+            {
+              result.stat = YP_TRUE;
+              if (mangle)
+                if (mangle_field (&rdat, mangle) < 0)
+                  result.stat = YP_YPERR;
+              result.val.valdat_len = rdat.dsize;
+              result.val.valdat_val = rdat.dptr;
+            }
+          else
+            result.stat = YP_NOKEY;
 
 	  ypdb_close (dbp);
 	}
@@ -351,7 +416,8 @@ ypproc_match_2_svc (ypreq_key * key, struct svc_req *rqstp)
 	  len *= j;
 	  if ((cp = malloc (len + 1)) == NULL)
 	    {
-	      yp_msg ("ERROR: could not allocate enough memory! [%s|%d]\n", __FILE__, __LINE__);
+	      yp_msg ("ERROR: could not allocate enough memory! [%s|%d]\n",
+		      __FILE__, __LINE__);
 	      result.stat = YP_YPERR;
 	      return &result;
 	    }
@@ -399,12 +465,12 @@ ypproc_match_2_svc (ypreq_key * key, struct svc_req *rqstp)
 }
 
 ypresp_key_val *
-ypproc_first_2_svc (ypreq_nokey * key, struct svc_req * rqstp)
+ypproc_first_2_svc (ypreq_nokey *key, struct svc_req *rqstp)
 {
   static ypresp_key_val result;
   struct sockaddr_in *rqhost;
   int valid;
-  int mangle_field = 0;
+  int mangle = 0;
 
   if (debug_flag)
     {
@@ -415,11 +481,20 @@ ypproc_first_2_svc (ypreq_nokey * key, struct svc_req * rqstp)
 
       yp_msg ("\tdomainname = \"%s\"\n", key->domain);
       yp_msg ("\tmapname = \"%s\"\n", key->map);
-#if 0
-      yp_msg ("\tkeydat = \"%.*s\"\n",
-	      (int) key->key.keydat_len,
-	      key->key.keydat_val);
-#endif
+    }
+
+  result.key.keydat_len = 0;
+  if (result.key.keydat_val)
+    {
+      ypdb_free (result.key.keydat_val);
+      result.key.keydat_val = NULL;
+    }
+
+  result.val.valdat_len = 0;
+  if (result.val.valdat_val)
+    {
+      ypdb_free (result.val.valdat_val);
+      result.val.valdat_val = NULL;
     }
 
   if ((valid = is_valid_host (rqstp, key->map, key->domain)) < 1)
@@ -439,44 +514,46 @@ ypproc_first_2_svc (ypreq_nokey * key, struct svc_req * rqstp)
       return &result;
     }
   else if (valid > 200)
-    mangle_field = valid - 200;
-
-  result.key.keydat_len = 0;
-  if (result.key.keydat_val)
-    {
-      free (result.key.keydat_val);
-      result.key.keydat_val = NULL;
-    }
-
-  result.val.valdat_len = 0;
-  if (result.val.valdat_val)
-    {
-      free (result.val.valdat_val);
-      result.val.valdat_val = NULL;
-    }
+    mangle = valid - 200;
 
   if (key->map[0] == '\0' || key->domain[0] == '\0')
     result.stat = YP_BADARGS;
   else
     {
-      datum dkey, dval;
-
       DB_FILE dbp = ypdb_open (key->domain, key->map);
       if (dbp == NULL)
-	result.stat = YP_NOMAP;
+        result.stat = YP_NOMAP;
       else
-	{
-	  result.stat = ypdb_read (dbp, NULL, &dkey, &dval, 0, mangle_field);
-
-	  if (result.stat == YP_TRUE)
+        {
+	  datum dkey = ypdb_firstkey (dbp);
+	  while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
 	    {
+#if defined(HAVE_NDBM) /* This is much more faster then ypdb_nextkey, but
+			  it is terrible to port to other databases */
+	      dkey = dbm_nextkey (dbp);
+#else
+	      datum tkey = dkey;
+	      dkey = ypdb_nextkey (dbp, tkey);
+	      ypdb_free (tkey.dptr);
+#endif
+	    }
+
+	  if (dkey.dptr != NULL)
+	    {
+	      datum dval = ypdb_fetch (dbp, dkey);
+	      result.stat = YP_TRUE;
+	      if (mangle)
+		if (mangle_field (&dval, mangle) < 0)
+		  result.stat = YP_YPERR;
+
 	      result.key.keydat_len = dkey.dsize;
 	      result.key.keydat_val = dkey.dptr;
 
 	      result.val.valdat_len = dval.dsize;
 	      result.val.valdat_val = dval.dptr;
 	    }
-
+	  else
+	    result.stat = YP_NOKEY;
 	  ypdb_close (dbp);
 	}
     }
@@ -500,12 +577,12 @@ ypproc_first_2_svc (ypreq_nokey * key, struct svc_req * rqstp)
 }
 
 ypresp_key_val *
-ypproc_next_2_svc (ypreq_key * key, struct svc_req * rqstp)
+ypproc_next_2_svc (ypreq_key *key, struct svc_req *rqstp)
 {
   static ypresp_key_val result;
   struct sockaddr_in *rqhost;
   int valid;
-  int mangle_field = 0;
+  int mangle = 0;
 
   if (debug_flag)
     {
@@ -522,6 +599,20 @@ ypproc_next_2_svc (ypreq_key * key, struct svc_req * rqstp)
 	      key->key.keydat_val);
     }
 
+  result.key.keydat_len = 0;
+  if (result.key.keydat_val)
+    {
+      ypdb_free (result.key.keydat_val);
+      result.key.keydat_val = NULL;
+    }
+
+  result.val.valdat_len = 0;
+  if (result.val.valdat_val)
+    {
+      ypdb_free (result.val.valdat_val);
+      result.val.valdat_val = NULL;
+    }
+
   if ((valid = is_valid_host (rqstp, key->map, key->domain)) < 1)
     {
       if (valid == 0)
@@ -539,21 +630,7 @@ ypproc_next_2_svc (ypreq_key * key, struct svc_req * rqstp)
       return &result;
     }
   else if (valid > 200)
-    mangle_field = valid - 200;
-
-  result.key.keydat_len = 0;
-  if (result.key.keydat_val)
-    {
-      free (result.key.keydat_val);
-      result.key.keydat_val = NULL;
-    }
-
-  result.val.valdat_len = 0;
-  if (result.val.valdat_val)
-    {
-      free (result.val.valdat_val);
-      result.val.valdat_val = NULL;
-    }
+    mangle = valid - 200;
 
   if (key->map[0] == '\0' || key->domain[0] == '\0')
     result.stat = YP_BADARGS;
@@ -561,41 +638,47 @@ ypproc_next_2_svc (ypreq_key * key, struct svc_req * rqstp)
     result.stat = YP_NODOM;
   else
     {
-      datum dkey, dval, okey;
-
       DB_FILE dbp = ypdb_open (key->domain, key->map);
       if (dbp == NULL)
-	result.stat = YP_NOMAP;
+        result.stat = YP_NOMAP;
       else
-	{
-	  dkey.dsize = key->key.keydat_len;
-	  dkey.dptr = key->key.keydat_val;
+        {
+          datum oldkey, dkey;
 
-	  result.stat = ypdb_read (dbp, &dkey, &okey, &dval,
-				 F_NEXT, mangle_field);
-	  /*
-	  ** If the key is empty, get the next one, until we
-	  ** get a key which is not empty
-	  */
-	  while ((okey.dsize == 0) && (result.stat == YP_TRUE))
+          oldkey.dsize = key->key.keydat_len;
+          oldkey.dptr = strndup (key->key.keydat_val, oldkey.dsize);
+
+          dkey = ypdb_nextkey (dbp, oldkey);
+	  while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
 	    {
-	      dkey.dsize = okey.dsize;
-	      dkey.dptr = okey.dptr;
-
-	      result.stat = ypdb_read (dbp, &dkey, &okey, &dval,
-				     F_NEXT, mangle_field);
+	      free (oldkey.dptr);
+	      oldkey.dsize = dkey.dsize;
+	      oldkey.dptr = strndup (dkey.dptr, dkey.dsize);
+	      ypdb_free (dkey.dptr);
+	      dkey = ypdb_nextkey (dbp, oldkey);
 	    }
 
-	  if (result.stat == YP_TRUE)
-	    {
-	      result.key.keydat_len = okey.dsize;
-	      result.key.keydat_val = okey.dptr;
+          free (oldkey.dptr);
 
-	      result.val.valdat_len = dval.dsize;
-	      result.val.valdat_val = dval.dptr;
-	    }
-	  ypdb_close (dbp);
-	}
+          if (dkey.dptr == NULL)
+            result.stat = YP_NOMORE;
+          else
+            {
+              datum dval = ypdb_fetch (dbp, dkey);
+
+              result.stat = YP_TRUE;
+              if (mangle)
+                if (mangle_field (&dval, mangle) < 0)
+                  result.stat = YP_YPERR;
+
+              result.key.keydat_len = dkey.dsize;
+              result.key.keydat_val = dkey.dptr;
+
+              result.val.valdat_len = dval.dsize;
+              result.val.valdat_val = dval.dptr;
+            }
+          ypdb_close (dbp);
+        }
     }
 
   if (debug_flag)
@@ -699,7 +782,7 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	{
 	  datum val = ypdb_fetch (dbp, key);
 
-	  if (val.dsize != strlen (xfr->map_parms.peer) ||
+	  if ((size_t)val.dsize != strlen (xfr->map_parms.peer) ||
 	      strncmp (val.dptr, xfr->map_parms.peer, val.dsize) != 0)
 	    {
 	      if (debug_flag)
@@ -722,6 +805,18 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 	}
       ypdb_close(dbp);
     }
+  /* If you wish to refuse the transfer of new maps, change the next
+     #if 0 statement to #if 1 */
+#if 0
+  else
+    {
+      /* We doesn't have the map, refuse the transfer */
+      if (debug_flag)
+	yp_msg ("\t->Ignored (I don't have this map)\n");
+      result.xfrstat = YPXFR_REFUSED;
+      return &result;
+    }
+#endif
 
   switch (fork ())
     {
@@ -758,15 +853,12 @@ ypproc_xfr_2_svc (ypreq_xfr *xfr, struct svc_req *rqstp)
 }
 
 
-/*
-** We don't need to do anything, since we doesn't cache anything.
-** Maybe we should ?
-*/
+/* Close all cached filehandles */
 void *
 ypproc_clear_2_svc (void *dummy, struct svc_req *rqstp)
 {
-  static char *result;
-  static char rval = 0;
+  static int *result;
+  static int rval;
 
   if (debug_flag)
     {
@@ -786,21 +878,19 @@ ypproc_clear_2_svc (void *dummy, struct svc_req *rqstp)
       return NULL;
     }
 
+  rval = ypdb_close_all ();
   result = &rval;
   return (void *) &result;
 }
 
-/*
-** We need the struct for giving ypall_encode the DB_FILE
-** handle and the mangle field.
-*/
+/* We need the struct for giving ypall_encode the DB_FILE
+   handle and the mangle field. */
 
 typedef struct ypall_data
 {
   DB_FILE dbm;
   int field;
-}
- *ypall_data_t;
+} *ypall_data_t;
 
 static int
 ypall_close (void *data)
@@ -812,52 +902,68 @@ ypall_close (void *data)
     }
 
   ypdb_close (((ypall_data_t) data)->dbm);
-  free (data);
+  ypdb_free (data);
   return 0;
 }
 
 static int
-ypall_encode (ypresp_key_val * val, void *data)
+ypall_encode (ypresp_key_val *val, void *data)
 {
-  datum dkey, dval, okey;
+  datum dkey;
 
-  dkey.dptr = val->key.keydat_val;
-  dkey.dsize = val->key.keydat_len;
+#if defined(HAVE_NDBM)
 
-  val->stat = ypdb_read (((ypall_data_t) data)->dbm, &dkey, &okey, &dval,
-		       F_NEXT, ((ypall_data_t) data)->field);
+  dkey = dbm_nextkey (((ypall_data_t) data)->dbm);
+  while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
+    dkey = dbm_nextkey (((ypall_data_t) data)->dbm);
 
-  /*
-  ** If key is empty, search the next one which is not empty
-  */
-  while ((okey.dsize == 0) && (val->stat == YP_TRUE))
+#else
+  datum oldkey;
+
+  oldkey.dsize = val->key.keydat_len;
+  oldkey.dptr = strndup (val->key.keydat_val, oldkey.dsize);
+
+  dkey = ypdb_nextkey (((ypall_data_t) data)->dbm, oldkey);
+  while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
     {
-      dkey.dptr = okey.dptr;
-      dkey.dsize = okey.dsize;
+      free (oldkey.dptr);
+      oldkey.dsize = dkey.dsize;
+      oldkey.dptr = strndup (dkey.dptr, dkey.dsize);
+      ypdb_free (dkey.dptr);
 
-      val->stat = ypdb_read (((ypall_data_t) data)->dbm, &dkey, &okey, &dval,
-			   F_NEXT, ((ypall_data_t) data)->field);
+      dkey = ypdb_nextkey (((ypall_data_t) data)->dbm, oldkey);
     }
 
-  if (val->stat == YP_TRUE)
+  free (oldkey.dptr);
+#endif
+
+  if (dkey.dptr == NULL)
+    val->stat = YP_NOMORE;
+  else
     {
-      val->key.keydat_val = okey.dptr;
-      val->key.keydat_len = okey.dsize;
+      datum dval = ypdb_fetch (((ypall_data_t) data)->dbm, dkey);
+
+      val->stat = YP_TRUE;
+      if (((ypall_data_t) data)->field)
+        if (mangle_field (&dval, ((ypall_data_t) data)->field) < 0)
+          val->stat = YP_YPERR;
+
+      val->key.keydat_val = dkey.dptr;
+      val->key.keydat_len = dkey.dsize;
 
       val->val.valdat_val = dval.dptr;
       val->val.valdat_len = dval.dsize;
     }
-
   return val->stat;
 }
 
 ypresp_all *
-ypproc_all_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
+ypproc_all_2_svc (ypreq_nokey *nokey, struct svc_req *rqstp)
 {
   static ypresp_all result;
   extern xdr_ypall_cb_t xdr_ypall_cb;
   int valid;
-  int mangle_field = 0;
+  int mangle = 0;
 
   if (debug_flag)
     {
@@ -890,7 +996,7 @@ ypproc_all_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
       return &result;
     }
   if (valid > 200)
-    mangle_field = valid - 200;
+    mangle = valid - 200;
 
   if (children < MAX_CHILDREN && fork ())
     {
@@ -910,12 +1016,12 @@ ypproc_all_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
     result.ypresp_all_u.val.stat = YP_BADARGS;
   else
     {
-      datum dkey, dval;
       ypall_data_t data;
 
       if ((data = malloc (sizeof (struct ypall_data))) == NULL)
 	{
-	  yp_msg ("ERROR: could not allocate enough memory! [%s|%d]\n", __FILE__, __LINE__);
+	  yp_msg ("ERROR: could not allocate enough memory! [%s|%d]\n",
+		  __FILE__, __LINE__);
 	  result.ypresp_all_u.val.stat = YP_YPERR;
 	  return &result;
 	}
@@ -925,36 +1031,52 @@ ypproc_all_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
 	result.ypresp_all_u.val.stat = YP_NOMAP;
       else
 	{
-	  result.ypresp_all_u.val.stat =
-	    ypdb_read (data->dbm, NULL, &dkey, &dval, 0, mangle_field);
-
-	  if (result.ypresp_all_u.val.stat == YP_TRUE)
+          datum dkey = ypdb_firstkey (data->dbm);
+	  while (dkey.dptr != NULL && strncmp (dkey.dptr, "YP_", 3) == 0)
 	    {
-	      result.ypresp_all_u.val.key.keydat_len = dkey.dsize;
-	      result.ypresp_all_u.val.key.keydat_val = dkey.dptr;
-
-	      result.ypresp_all_u.val.val.valdat_len = dval.dsize;
-	      result.ypresp_all_u.val.val.valdat_val = dval.dptr;
-
-	      data->field = mangle_field;
-
-	      xdr_ypall_cb.u.encode = ypall_encode;
-	      xdr_ypall_cb.u.close = ypall_close;
-	      xdr_ypall_cb.data = (void *) data;
-
-	      if (debug_flag)
-		yp_msg ("\t -> First value returned.\n");
-
-	      return &result;
+	      datum tkey = dkey;
+	      dkey = ypdb_nextkey (data->dbm, tkey);
+	      ypdb_free (tkey.dptr);
 	    }
 
+          if (dkey.dptr != NULL)
+            {
+              datum dval = ypdb_fetch (data->dbm, dkey);
+
+              result.ypresp_all_u.val.stat = YP_TRUE;
+
+              if (mangle)
+                if (mangle_field (&dval, mangle) < 0)
+                  result.ypresp_all_u.val.stat = YP_YPERR;
+
+              result.ypresp_all_u.val.key.keydat_len = dkey.dsize;
+              result.ypresp_all_u.val.key.keydat_val = dkey.dptr;
+
+              result.ypresp_all_u.val.val.valdat_len = dval.dsize;
+              result.ypresp_all_u.val.val.valdat_val = dval.dptr;
+
+              data->field = mangle;
+
+              xdr_ypall_cb.u.encode = ypall_encode;
+              xdr_ypall_cb.u.close = ypall_close;
+              xdr_ypall_cb.data = (void *) data;
+
+              if (debug_flag)
+                yp_msg ("\t -> First value returned.\n");
+
+              if (result.ypresp_all_u.val.stat == YP_TRUE)
+                return &result;
+            }
+	  else
+	    result.ypresp_all_u.val.stat = YP_NOMORE;
+
 	  ypdb_close (data->dbm);
 	}
       free (data);
     }
 
   if (debug_flag)
-    yp_msg ("\t -> Error should have been returned.\n");
+    yp_msg ("\t -> Exit from ypproc_all without sending data.\n");
 
   return &result;
 }
@@ -979,6 +1101,12 @@ ypproc_master_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
       yp_msg ("\t\tmap = \"%s\"\n", nokey->map);
     }
 
+  if (result.peer)
+    {
+      free (result.peer);
+      result.peer = NULL;
+    }
+
   if ((valid = is_valid_host (rqstp, nokey->map, nokey->domain)) < 1)
     {
       if (valid == 0)
@@ -997,12 +1125,6 @@ ypproc_master_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
       return &result;
     }
 
-  if (result.peer)
-    {
-      free (result.peer);
-      result.peer = NULL;
-    }
-
   if (nokey->map[0] == '\0')
     result.stat = YP_BADARGS;
   else
@@ -1021,7 +1143,26 @@ ypproc_master_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
 	  if (val.dptr == NULL)
 	    {
 	      /* No YP_MASTER_NAME record in map? Assume we are Master */
-	      gethostname (hostbuf, sizeof (hostbuf) - 1);
+	      if(gethostname (hostbuf, sizeof (hostbuf) - 1) != 0)
+		{
+		  perror ("gethostname");
+		  exit (1);
+		}
+#if USE_FQDN
+	      else
+		{
+		  struct hostent *hp;
+
+		  if (!(hp = res_gethostbyname (hostbuf)))
+		    perror ("gethostbyname()");
+		  else
+		    {
+		      strncpy (hostbuf, hp->h_name, sizeof (hostbuf) - 1);
+		      hostbuf[sizeof (hostbuf) - 1] = '\0';
+	}
+    }
+#endif
+
 	      if ((result.peer = strdup (hostbuf)) == NULL)
 		result.stat = YP_YPERR;
 	      else
@@ -1035,7 +1176,7 @@ ypproc_master_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
 	      for (i = 0; i < val.dsize; i++)
 		hostbuf[i] = val.dptr[i];
 	      hostbuf[val.dsize] = '\0';
-	      free (val.dptr);
+	      ypdb_free (val.dptr);
 
 	      if ((result.peer = strdup (hostbuf)) == NULL)
 		result.stat = YP_YPERR;
@@ -1122,7 +1263,7 @@ ypproc_order_2_svc (ypreq_nokey * nokey, struct svc_req * rqstp)
 	      memcpy (buf, val.dptr, val.dsize);
 	      buf[val.dsize + 1] = '\0';
 	      result.ordernum = atoi (buf);
-	      free (val.dptr);
+	      ypdb_free (val.dptr);
 	    }
 
 	  result.stat = YP_TRUE;
@@ -1154,6 +1295,16 @@ static int
 add_maplist (ypmaplist ** mlhp, char *map)
 {
   ypmaplist *mlp;
+#if defined(HAVE_NDBM)
+#if defined(sun) || defined(__sun__)
+  /* XXX We have real NDBM support. Need some more hacks here */
+#else
+  int len = strlen (map);
+
+  if (map[len - 3] == '.' && map[len - 2] == 'd' && map[len - 1] == 'b')
+    map[len - 3] = '\0';
+#endif
+#endif
 
   if ((mlp = malloc (sizeof (*mlp))) == NULL)
     return -1;
