@@ -1,4 +1,4 @@
-/* $Id: system.h,v 1.8 1997/01/30 18:53:23 kukuk Exp $ */
+/* $Id: system.h,v 1.12 1998/03/06 20:01:11 kukuk Exp $ */
 #ifndef __SYSTEM_H__
 #define __SYSTEM_H__
 
@@ -58,6 +58,7 @@ extern pid_t wait4(pid_t pid, int *status, int options, struct rusage *rusage);
 #endif
 
 #if (defined(__sun__) || defined(sun)) && defined(__svr4__)
+/* Solaris 2.x */
 
 #define __EXTENSIONS__
 #define NEED_SVCSOC_H
@@ -98,12 +99,16 @@ extern void svcerr_systemerr();
 #define _BSD_SOURCE
 #endif
 
-extern int snprintf (char *, unsigned int, const char *, ...);
+#include <stdio.h>
 
+#if !(defined (__GLIBC__) && __GLIBC__ >= 2)
+extern int snprintf (char *, unsigned int, const char *, ...);
 #endif
 
+#endif /* linux */
+
 #if !defined(HAVE_RES_GETHOSTBYNAME)
-#if defined(HAVE__DNS_GETHOSTBYNAME)
+#if defined(HAVE__DNS_GETHOSTBYNAME) || defined(linux)
 #define res_gethostbyname _dns_gethostbyname
 extern struct hostent *_dns_gethostbyname(const char *);
 #define res_gethostbyaddr _dns_gethostbyaddr
